<?php

namespace Drupal\autoban\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure autoban settings for this site.
 */
class AutobanSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'autoban_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'autoban.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('autoban.settings');

    $thresholds = $config->get('autoban_thresholds');
    if (empty($thresholds)) {
      $thresholds = "1\n2\n3\n5\n10\n20\n50\n100";
    }
    $form['autoban_thresholds'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Thresholds'),
      '#default_value' => $thresholds,
      '#required' => TRUE,
      '#description' => $this->t('Thresholds set for Autoban rules threshold field.'),
    ];

    $query_mode = $config->get('autoban_query_mode');
    if (empty($query_mode)) {
      $query_mode = 'like';
    }
    $form['autoban_query_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Query mode'),
      '#options' => ['like' => 'LIKE', 'regexp' => 'REGEXP'],
      '#default_value' => $query_mode,
      '#description' => $this->t('Use REGEXP option if your SQL engine supports REGEXP syntax.'),
    ];


    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $thresholds = explode("\n", $form_state->getValue('autoban_thresholds'));
    foreach ($thresholds as $threshold) {
      $threshold = trim($threshold);
      if (!is_numeric($threshold)) {
        $form_state->setErrorByName('autoban_thresholds', $this->t('Threshold value (@value) must be an integer.', ['@value' => $threshold]));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $thresholds = explode("\n", $form_state->getValue('autoban_thresholds'));
    array_walk($thresholds, function(&$item, $key) {
      $item = (int)$item;
    });

    \Drupal::configFactory()->getEditable('autoban.settings')
      ->set('autoban_thresholds', implode("\n", $thresholds))
       ->set('autoban_query_mode', $form_state->getValue('autoban_query_mode'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
